/* Copyright (C) 2006 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef _STORAGE_HANDLER_H_
#define _STORAGE_HANDLER_H_

#include "SyncObject.h"

static const int connectionHashSize = 101;

enum OpenOption {
	CreateDatabase,
	OpenDatabase,
	OpenOrCreateDatabase,
	OpenTemporaryDatabase
	};
	

typedef void (Logger) (int, const char*, void *arg);

struct IOAnalysis;
class StorageConnection;
class StorageHandler;
class StorageDatabase;
class StorageTableShare;
class SyncObject;
class Connection;
class InfoTable;
class PreparedStatement;
class THD;

extern "C" 
{
StorageHandler*	getFalconStorageHandler(int lockSize);
}

static const int databaseHashSize = 101;
static const int tableHashSize = 101;

class StorageHandler
{
public:
	StorageHandler(int lockSize);
	virtual ~StorageHandler(void);
	virtual void		startNfsServer(void);
	virtual void		addNfsLogger(int mask, Logger listener, void* arg);
	//virtual int		isTempTable(const char *path);
	//virtual StorageConnection* getStorageConnection(const char* dbName, THD* mySqlThread, OpenOption createFlag, const char *path, const char *tableSpace);

	virtual void		shutdownHandler(void);
	virtual void		databaseDropped(StorageDatabase *storageDatabase, StorageConnection* storageConnection);
	virtual int			commit(THD* mySqlThread);
	virtual int			rollback(THD* mySqlThread);
	virtual int			releaseVerb(THD* mySqlThread);
	virtual int			rollbackVerb(THD* mySqlThread);
	virtual int			savepointSet(THD* mySqlThread, void* savePoint);
	virtual int			savepointRelease(THD* mySqlThread, void* savePoint);
	virtual int			savepointRollback(THD* mySqlThread, void* savePoint);
	//virtual const char* getEngineStatus(THD* mySqlThread);
	virtual void		releaseText(const char* text);
	virtual int			commitByXID(int xidLength, const unsigned char* xid);
	virtual int			rollbackByXID(int xidLength, const unsigned char* xis);
	virtual Connection*	getDictionaryConnection(void);
	virtual int			createTablespace(const char* tableSpaceName, const char* filename);
	virtual int			deleteTablespace(const char* tableSpaceName);

	virtual StorageTableShare* findTable(const char* pathname);
	virtual StorageTableShare* createTable(const char* pathname, const char *tableSpaceName, bool tempTable);
	virtual StorageConnection* getStorageConnection(StorageTableShare* tableShare, THD* mySqlThread, int mySqlThdId, OpenOption createFlag);

	virtual void		getIOInfo(InfoTable* infoTable);
	virtual void		getMemoryDetailInfo(InfoTable* infoTable);
	virtual void		getMemorySummaryInfo(InfoTable* infoTable);
	virtual void		getRecordCacheDetailInfo(InfoTable* infoTable);
	virtual void		getRecordCacheSummaryInfo(InfoTable* infoTable);
	virtual void		getTransactionInfo(InfoTable* infoTable);
	virtual void		getSerialLogInfo(InfoTable* infoTable);
	virtual void		getSyncInfo(InfoTable* infoTable);
	virtual void		getTransactionSummaryInfo(InfoTable* infoTable);

	StorageDatabase*	getStorageDatabase(const char* dbName, const char* path);
	void				remove(StorageConnection* storageConnection);
	void				closeDatabase(const char* path);
	int					prepare(THD* mySqlThread, int xidSize, const unsigned char *xid);
	StorageDatabase*	findTablespace(const char* name);
	void				removeTable(StorageTableShare* table);
	void				addTable(StorageTableShare* table);
	StorageDatabase*	findDatabase(const char* dbName);
	void				changeMySqlThread(StorageConnection* storageConnection, THD* newThread);
	void				removeConnection(StorageConnection* storageConnection);
	int					closeConnections(THD* thd);
	int					dropDatabase(const char* path);
	
	StorageConnection	*connections[connectionHashSize];
	SyncObject			syncObject;
	SyncObject			hashSyncObject;
	SyncObject			dictionarySyncObject;
	StorageDatabase		*storageDatabases[databaseHashSize];
	StorageDatabase		*databaseList;
	StorageTableShare	*tables[tableHashSize];
	Connection			*dictionaryConnection;
	int					mySqlLockSize;
	bool				globalTableSpace;
};

#endif
